@@ -48,7 +48,7 @@ class GitLibMixin:
         "ServerTribe Attune", "attune@servertribe.com"
     )
 
-    def __init__(self, projectPath: Path):
+    def __init__(self, projectPath: Path, projectInfo: "ContextProjectInfo"):
         self.projectPath = projectPath
         self.__committer = self.defaultCommitter
         self._repo = None
@@ -62,7 +62,7 @@ class GitLibMixin:
         try:
             self.loadGitRepo()
         except pygit2.GitError:
-            self.initGitRepo()
+            self.initGitRepo(projectInfo)
             self.loadGitRepo()
 
     @property
@@ -78,32 +78,36 @@ class GitLibMixin:
         if self._repo.is_bare:
             raise ValueError(f"{self._repoPath} points to a Git Bare Repo")
 
-    def initGitRepo(self):
+    def initGitRepo(self, projectInfo):
         # Create target repo
         self._repo = pygit2.init_repository(self._repoPath.as_posix())
         self.__indexInitialised = True
         logger.info(f"Create Git repository at {self._repoPath}")
-        self.__createInitialCommit()
+        self.__createInitialCommit(projectInfo)
 
-    def __createInitialCommit(self):
+    def __createInitialCommit(self, projectInfo):
         # We cannot use the public `writeMetadata` method here because the
         # branches and references to have not been set up yet
-        self.__writeProjectMetadata()
+        self.__writeInitialProjectMetadata(projectInfo)
         self.__writeGitIgnore()
         self.__createCommit(
             "HEAD", "Create default metadata.json", [], self.__index
         )
 
-    def __writeProjectMetadata(self):
+    def __writeInitialProjectMetadata(self, projectInfo: "ContextProjectInfo"):
         # Create default metadata.json
-        name = self.project.name
+        name = projectInfo.name
         data = json.dumps(
-            {"revision": getLatestRevision(), "name": name},
+            {
+                "revision": getLatestRevision(),
+                "name": projectInfo.name,
+                "key": projectInfo.key,
+            },
             indent=4,
             sort_keys=True,
             separators=(", ", ": "),
         ).encode()
-        path = "metadata.json"
+        path = "project/metadata.json"
         # Write the file, *without* preparing the working branch
         self.__index.add(
             pygit2.IndexEntry(
@@ -123,30 +127,7 @@ class GitLibMixin:
         )
         self.__isIndexDirty = True
 
-    @property
-    def metadata(self):
-        tree = self.__workingTree
-        data = (tree / "metadata.json").data
-        metadataStr = data.decode()
-        metadata = json.loads(metadataStr)
-        return metadata
-
-    def writeMetadata(self, metadata):
-        oid = self._repo.create_blob(
-            json.dumps(
-                metadata,
-                indent=4,
-                sort_keys=True,
-                separators=(", ", ": "),
-            ).encode()
-        )
-        self.prepareWorkingBranch()
-        self.__index.add(
-            pygit2.IndexEntry("metadata.json", oid, pygit2.GIT_FILEMODE_BLOB)
-        )
-        self.__isIndexDirty = True
-
-    def _writeReadme(self, content: str):
+    def writeReadme(self, content: str):
         oid = self._repo.create_blob(content.encode())
         # Ensure there is a __working__ branch present
         self.prepareWorkingBranch()
@@ -374,6 +355,9 @@ class GitLibMixin:
         return self._repo.revparse_single(branch).tree
 
     def _getTree(self, path: Path) -> pygit2.Tree:
+        if path.as_posix().startswith("./"):
+            raise PathNameInvalidException("Paths should not be relative")
+
         # We will read objects as they are at the working branch
         tree = self.__workingTree
         climbedPath = ""
@@ -512,7 +496,7 @@ class GitLibMixin:
     def _generateReadme(self) -> None:
         readmeCompiler = ReadmeCompiler(self)
         readmeContent = readmeCompiler.compile()
-        self._writeReadme(readmeContent)
+        self.writeReadme(readmeContent)
         self.commit("Updated README.md")
 
     @property
@@ -693,11 +677,14 @@ class GitLibMixin:
                     )
 
         checkedOutBranch = self.__checkedOutBranch
-        remoteBranchName = f"{remoteName}/{checkedOutBranch.shorthand}"
-        remoteBranch = self._repo.branches[remoteBranchName]
 
-        if checkedOutBranch.target == remoteBranch.target:
-            raise Exception("There are no commits to push")
+        # The remote branch might not exist when a new repository is
+        # created
+        remoteBranchName = f"{remoteName}/{checkedOutBranch.shorthand}"
+        if remoteBranchName in self._repo.branches:
+            remoteBranch = self._repo.branches[remoteBranchName]
+            if checkedOutBranch.target == remoteBranch.target:
+                raise Exception("There are no commits to push")
 
         # TODO: This does not seem to raise any errors when a push was
         #  unsuccessful due to permission errors
@@ -733,8 +720,10 @@ class GitLibMixin:
 
         checkedOutBranch = self.__checkedOutBranch
         remoteBranchName = f"{remoteName}/{checkedOutBranch.shorthand}"
-        # Try to merge remote equivalent into current checked out
-        self.mergeBranches(remoteBranchName)
+
+        if remoteBranchName in self._repo.branches:
+            # Try to merge remote equivalent into current checked out
+            self.mergeBranches(remoteBranchName)
 
     def mergeBranches(self, sourceBranch: str) -> None:
         if sourceBranch not in self.allBranches():
@@ -845,7 +834,7 @@ class GitLibMixin:
         else:
             logger.debug(
                 f"Something went wrong. Resetting __working__ to"
-                f" {self._workingId} in {self.project.name}"
+                f" {self._workingId}"
             )
             checkedBranchName = self._checkedOutBranchRef
             self._repo.checkout(makeRef(ATTUNE_WORKING_BRANCH))
@@ -865,18 +854,5 @@ class GitLibMixin:
             a=self.__checkedOutBranch.target, b=self.__workingBranch.target
         )
 
-    def getCommits(self) -> list[GitCommitTuple]:
-        for commit in self._repo.walk(self.__checkedOutBranch.target):
-            gitCommit = GitCommitTuple()
-            gitCommit.authorName = commit.author.name
-            gitCommit.authorEmail = commit.author.email
-            gitCommit.message = commit.message
-            gitCommit.timestamp = datetime.fromtimestamp(
-                commit.commit_time
-            ).strftime("%Y-%m-%d %H:%M:%S")
-            gitCommit.hash = commit.short_id
-
-            yield gitCommit
-
     def freeGitDbResources(self) -> None:
         self._repo.free()
