@@ -14,6 +14,7 @@ from twisted.internet.defer import DeferredSemaphore
 from twisted.internet.defer import inlineCallbacks
 from vortex.Tuple import Tuple
 
+from attune_project_api.items.project_metadata_tuple import ProjectMetadataTuple
 from attune_project_api.items.step_tuples.step_sql_oracle_tuple import (
     StepSqlOracleTuple,
 )
@@ -23,7 +24,6 @@ from .GitLibMixin import GitLibMixin
 from ..Exceptions import ItemNotFoundError
 from ..Exceptions import ProjectIncorrectVersionError
 from ..Exceptions import ProjectValidationError
-from ..ObjectStorageContext import ContextProjectInfo
 from ..ObjectStorageContext import ObjectStorageContext
 from ..ObjectStorageContext import ObjectStorageContextFileMixin
 from ..ObjectStorageContext import VersionedFileContent
@@ -32,6 +32,7 @@ from ..StorageTuple import ItemStorageGroupEnum
 from ..StorageTuple import StorageTuple
 from .._private.large_file_http_downloader import LargeFileHttpDownloader
 from .._private.utils import makeStorageKey
+from ..context_project_info import ContextProjectInfo
 from ..items.file_archive_tuples.file_tuple_constants import (
     FileArchiveTupleTypeEnum,
 )
@@ -266,27 +267,31 @@ class GitObjectStorageContext(
 ):
     def __init__(self, projectPath: Path, info: ContextProjectInfo):
         assert isinstance(projectPath, Path), "projectPath is not of type Path"
-        self.__projectInfo = info
-        GitLibMixin.__init__(self, projectPath)
-
-        revision = self.getRevision()
-        if not checkIfLibrarySupportsRevision(revision):
-            raise ProjectIncorrectVersionError(
-                "A newer version of Attune is "
-                f"required to load the {info.name} project"
-            )
+        self._projectInfo = info
+        GitLibMixin.__init__(self, projectPath, info)
 
         """ This is equivalent to an Identity Map in SqlAlchemy Session
-        Holds references to all instances fetched from the storage
-        ensuring that "one and only one" reference exists for an item
-        For example, s1 = getItem("Step1"), s2 = getItem("Step1")
-        id(s1) == id(s2) """
+                Holds references to all instances fetched from the storage
+                ensuring that "one and only one" reference exists for an item
+                For example, s1 = getItem("Step1"), s2 = getItem("Step1")
+                id(s1) == id(s2) """
         self.__instanceByKeyByGroup = defaultdict(dict)
 
         self.__instanceByKeyByGroup[ItemStorageGroupEnum.Step] = {}
         self.__instanceByKeyByGroup[ItemStorageGroupEnum.Parameter] = {}
         self.__instanceByKeyByGroup[ItemStorageGroupEnum.FileArchive] = {}
 
+        self.__singularInstanceGroup = defaultdict(lambda: None)
+        self.__singularInstanceGroup[ItemStorageGroupEnum.Project] = None
+        self.__loadSingularItem(ItemStorageGroupEnum.Project)
+
+        revision = self.metadata.revision
+        if not checkIfLibrarySupportsRevision(revision):
+            raise ProjectIncorrectVersionError(
+                "A newer version of Attune is "
+                f"required to load the {info.name} project"
+            )
+
     def load(self):
         # Run the migrations to bring the project to the latest revision
         # Migrations need to be run before the items are loaded and as a
@@ -304,26 +309,48 @@ class GitObjectStorageContext(
         self.__instanceByKeyByGroup[ItemStorageGroupEnum.Step] = {}
         self.__instanceByKeyByGroup[ItemStorageGroupEnum.Parameter] = {}
         self.__instanceByKeyByGroup[ItemStorageGroupEnum.FileArchive] = {}
+        self.__singularInstanceGroup[ItemStorageGroupEnum.Project] = None
 
         self.__loadItems(ItemStorageGroupEnum.Parameter)
         self.__loadItems(ItemStorageGroupEnum.FileArchive)
         self.__loadItems(ItemStorageGroupEnum.Step)
 
+        self.__loadSingularItem(ItemStorageGroupEnum.Project)
+
     @property
-    def project(self) -> ContextProjectInfo:
-        return self.__projectInfo
+    def metadata(self) -> ProjectMetadataTuple:
+        metadata = self.__singularInstanceGroup[ItemStorageGroupEnum.Project]
+        if metadata is None:
+            metadata = self.__metadataLoadFallback()
+        return metadata
 
-    def getRevision(self) -> str:
-        metadata = self.metadata
-        # If revision is missing return the first revision
-        return metadata.get("revision", "16e34a42a9a1")
+    def __metadataLoadFallback(self):
+        metadata = ProjectMetadataTuple(
+            key="", name="", revision="", comment=""
+        )
+        try:
+            metadataFile = json.loads(self._readFile(Path("metadata.json")))
+        except FileNotFoundError:
+            try:
+                metadataFile = json.loads(
+                    self._readFile(Path("project/metadata.json"))
+                )
+            except FileNotFoundError:
+                metadataFile = {}
 
-    def setRevision(self, revision: str):
-        metadata = self.metadata
-        metadata["revision"] = revision
+        if "name" in metadataFile:
+            metadata.name = metadataFile["name"]
+            metadata.key = makeStorageKey(metadata.name)
+        else:
+            metadata.name = self._projectInfo.name
+            metadata.key = self._projectInfo.key
 
-        self.writeMetadata(metadata)
-        self.commit("Wrote metadata.json")
+        if "revision" in metadataFile:
+            metadata.revision = metadataFile["revision"]
+        else:
+            metadata.revision = "16e34a42a9a1"  # First revision
+
+        return metadata
 
     def getItem(
         self, group: ItemStorageGroupEnum, key: str
@@ -340,6 +367,15 @@ class GitObjectStorageContext(
 
         return itemsByKey[key]
 
+    def getSingularItem(
+        self, group: ItemStorageGroupEnum
+    ) -> Optional[StorageTuple]:
+        assert group in self.__singularInstanceGroup, (
+            "Group %s is not a valid" % group
+        )
+
+        return self.__singularInstanceGroup[group]
+
     def getItems(
         self,
         group: ItemStorageGroupEnum,
@@ -393,7 +429,10 @@ class GitObjectStorageContext(
 
     def mergeItem(self, item: StorageTuple) -> StorageTuple:
         try:
-            existingItem = self.getItem(item.storageGroup, item.key)
+            if item.__allowsMultiple__:
+                existingItem = self.getItem(item.storageGroup, item.key)
+            else:
+                existingItem = self.getSingularItem(item.storageGroup)
 
             # If it's the same item we have in our store, then there is
             # nothing to do.
@@ -461,10 +500,16 @@ class GitObjectStorageContext(
         assert (
             item.storageContext is self
         ), "Item must be bound before it is cached"
-        self.__instanceByKeyByGroup[item.storageGroup][item.key] = item
+        if item.__allowsMultiple__:
+            self.__instanceByKeyByGroup[item.storageGroup][item.key] = item
+        else:
+            self.__singularInstanceGroup[item.storageGroup] = item
 
     def __removeReferenceToTuple(self, item):
-        self.__instanceByKeyByGroup[item.storageGroup].pop(item.key, None)
+        if item.__allowsMultiple__:
+            self.__instanceByKeyByGroup[item.storageGroup].pop(item.key, None)
+        else:
+            self.__singularInstanceGroup.pop(item.storageGroup)
 
     def __loadItems(self, group: ItemStorageGroupEnum) -> None:
         startTime = datetime.now(pytz.utc)
@@ -474,6 +519,9 @@ class GitObjectStorageContext(
             return
 
         BaseClass = GitObjectStorageContext._BaseClassByGroup[group]
+        assert BaseClass.__allowsMultiple__, (
+            f"Items of {group.value} " f"contain a single instance"
+        )
 
         errors = []
         for object_tree in tree:
@@ -502,6 +550,26 @@ class GitObjectStorageContext(
             datetime.now(pytz.utc) - startTime,
         )
 
+    def __loadSingularItem(self, group: ItemStorageGroupEnum) -> None:
+        try:
+            tree = self._getTree(Path(group.value))
+        except FileNotFoundError:
+            return None
+
+        BaseClass = GitObjectStorageContext._BaseClassByGroup[group]
+        assert not BaseClass.__allowsMultiple__, (
+            f"Items of {group.value} " f"contain multiple instances"
+        )
+
+        try:
+            item = self.__readItem(BaseClass)
+        except Exception as e:
+            logger.debug(f"Error in loading group {group.value}: {str(e)}")
+            return None
+
+        item._bind(self)
+        self.__storeReferenceToTuple(item)
+
     def __writeItem(self, item: type[StorageTuple]) -> None:
         fields = item.tupleToRestfulJsonDict()
         comment = fields.pop("comment", None)
@@ -512,7 +580,11 @@ class GitObjectStorageContext(
         #  then an exception needs raising instead.
         sanitizedItemKey = makeStorageKey(item.key)
         group = item.storageGroup.value
-        itemPath = Path(f"{group}/{sanitizedItemKey}")
+
+        if not item.__allowsMultiple__:
+            itemPath = Path(f"{group}")
+        else:
+            itemPath = Path(f"{group}/{sanitizedItemKey}")
 
         self._writeFile(
             (itemPath / "metadata.json").as_posix(),
@@ -526,11 +598,11 @@ class GitObjectStorageContext(
 
         if comment:
             self._writeFile(
-                (itemPath / "comment.md").as_posix(), comment.encode()
+                (itemPath / "README.md").as_posix(), comment.encode()
             )
         else:
             try:
-                self._deleteFile(itemPath / "comment.md")
+                self._deleteFile(itemPath / "README.md")
             except (
                 KeyError,
                 OSError,
@@ -577,10 +649,19 @@ class GitObjectStorageContext(
         self._deleteDirectory(itemPath)
 
     def __readItem(
-        self, ItemClass: type[StorageTuple], key: str
+        self, ItemClass: type[StorageTuple], key: str = None
     ) -> StorageTuple:
         groupStr = ItemClass.storageGroup.value
-        path = Path(f"{groupStr}/{makeStorageKey(key)}")
+        if ItemClass.__allowsMultiple__:
+            assert key, f"Expected a key for reading item {groupStr}"
+            path = Path(f"{groupStr}/{makeStorageKey(key)}")
+        else:
+            if key:
+                logger.debug(
+                    f"Unexpected key {key} to readItem for group {groupStr}"
+                )
+            path = Path(f"{groupStr}")
+
         try:
             data = self._readFile(path / "metadata.json")
             metadataStr = data.decode()
@@ -589,15 +670,10 @@ class GitObjectStorageContext(
             raise ItemNotFoundError(f"{key} does not exist in {groupStr}")
 
         try:
-            commentData = self._readFile(path / "comment.md")
-            if commentData is None:
-                # Every item has an comment associated with it. If it does not
-                # exist we need to set it to None
-                metadata["comment"] = None
-            else:
-                metadata["comment"] = commentData.decode()
+            commentData = self._readFile(path / "README.md")
+            metadata["comment"] = commentData.decode()
         except FileNotFoundError:
-            metadata["comment"] = ""
+            metadata["comment"] = None  # Most tuples default this anyways
 
         try:
             scriptData = self._readFile(path / "script.txt")
@@ -647,9 +723,10 @@ class GitObjectStorageContext(
             )
 
         metadata = self.metadata
-        metadata["name"] = newName
+        metadata.name = newName
+        metadata.key = makeStorageKey(newName)
 
-        self.writeMetadata(metadata)
+        self.mergeItem(metadata)
         self.commit("Write metadata with new name")
         self.squashAndMergeWorking(f"Renamed project to {newName}")
 
